@@ -1,5 +1,5 @@
 /*
- * $Id: cond.c,v 2.4 1996/06/28 02:05:24 hzoli Exp $
+ * $Id: cond.c,v 2.6 1996/10/15 20:16:35 hzoli Exp $
  *
  * cond.c - evaluate conditional expressions
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -107,6 +107,8 @@ evalcond(Cond c)
 	return ((st = getstat(c->left)) && st->st_uid == geteuid());
     case 'G':
 	return ((st = getstat(c->left)) && st->st_gid == getegid());
+    case 'N':
+	return ((st = getstat(c->left)) && st->st_atime <= st->st_mtime);
     case 't':
 	return isatty(matheval(c->left));
     case COND_EQ:
