@@ -1,5 +1,5 @@
 /*
- * $Id: exec.c,v 2.84 1996/08/15 10:39:14 hzoli Exp $
+ * $Id: exec.c,v 2.93 1996/10/15 20:16:35 hzoli Exp $
  *
  * exec.c - command execution
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -31,8 +31,8 @@
 
 #include "zsh.h"
 
-#define execerr() { if (forked) _exit(1); \
-	closemnodes(mfds); lastval = 1; return; }
+#define execerr() if (!forked) { lastval = 1; return; } else _exit(1)
+	
 
 static LinkList args;
 static int doneps4;
@@ -46,7 +46,7 @@ parse_string(char *s)
     List l;
 
     lexsave();
-    inpush(s, 0);
+    inpush(s, 0, NULL);
     strinbeg();
     stophist = 2;
     l = parse_list();
@@ -119,18 +119,6 @@ static pid_t list_pipe_pid;
 static int nowait, pline_level = 0;
 static int list_pipe_child = 0, list_pipe_job;
 static char list_pipe_text[JOBTEXTSIZE];
-static LinkList last_file_list;
-
-/**/
-void
-deletepipejob(void)
-{
-    if (!list_pipe) {
-	last_file_list = jobtab[thisjob].filelist;
-	jobtab[thisjob].filelist = NULL;
-	deletejob(jobtab + thisjob);
-    }
-}
 
 /* execute a current shell command */
 
@@ -138,7 +126,8 @@ deletepipejob(void)
 int
 execcursh(Cmd cmd)
 {
-    deletepipejob();
+    if (!list_pipe)
+	deletejob(jobtab + thisjob);
     execlist(cmd->u.list, 1, cmd->flags & CFLAG_EXEC);
     cmd->u.list = NULL;
     return lastval;
@@ -266,7 +255,7 @@ execute(Cmdnam not_used_yet, int dash)
     /* If the parameter STTY is set in the command's environment, *
      * we first run the stty command with the value of this       *
      * parameter as it arguments.                                 */
-    if (!exargs && (s = zgetenv("STTY"))) {
+    if (!exargs && (s = zgetenv("STTY")) && isatty(0)) {
 	char *t;
 
 	exargs = args;	/* this prevents infinite recursion */
@@ -318,7 +307,7 @@ execute(Cmdnam not_used_yet, int dash)
 	    strcpy(nn, cn->u.cmd);
 	else {
 	    for (pp = path; pp < cn->u.name; pp++)
-		if (**pp == '.' && (*pp)[1] == '\0') {
+		if (!**pp || (**pp == '.' && (*pp)[1] == '\0')) {
 		    ee = zexecve(arg0, argv);
 		    if (isgooderr(ee, *pp))
 			eno = ee;
@@ -343,7 +332,7 @@ execute(Cmdnam not_used_yet, int dash)
 	    eno = ee;
     }
     for (pp = path; *pp; pp++)
-	if ((*pp)[0] == '.' && !(*pp)[1]) {
+	if (!(*pp)[0] || ((*pp)[0] == '.' && !(*pp)[1])) {
 	    ee = zexecve(arg0, argv);
 	    if (isgooderr(ee, *pp))
 		eno = ee;
@@ -397,8 +386,10 @@ findcmd(char *arg0)
 	    for (pp = path; pp < cn->u.name; pp++)
 		if (**pp != '/') {
 		    z = buf;
-		    strucpy(&z, *pp);
-		    *z++ = '/';
+		    if (**pp) {
+			strucpy(&z, *pp);
+			*z++ = '/';
+		    }
 		    strcpy(z, arg0);
 		    try(buf);
 		}
@@ -410,8 +401,10 @@ findcmd(char *arg0)
     }
     for (pp = path; *pp; pp++) {
 	z = buf;
-	strucpy(&z, *pp);
-	*z++ = '/';
+	if (**pp) {
+	    strucpy(&z, *pp);
+	    *z++ = '/';
+	}
 	strcpy(z, arg0);
 	try(buf);
     }
@@ -463,7 +456,7 @@ hashcmd(char *arg0, char **pp)
 		break;
 	}
 
-    if (!*pp || isrelative(*pp))
+    if (!*pp)
 	return NULL;
 
     cn = (Cmdnam) zcalloc(sizeof *cn);
@@ -632,9 +625,6 @@ execpline(Sublist l, int how, int last1)
     if (how & Z_TIMED)
 	jobtab[thisjob].stat |= STAT_TIMED;
 
-    jobtab[newjob].filelist = last_file_list;
-    last_file_list = NULL;
-
     if (l->flags & PFLAG_COPROC) {
 	how = Z_ASYNC;
 	if (coprocin >= 0) {
@@ -645,6 +635,7 @@ execpline(Sublist l, int how, int last1)
 	mpipe(opipe);
 	coprocin = ipipe[0];
 	coprocout = opipe[1];
+	fdtable[coprocin] = fdtable[coprocout] = 0;
     }
     if (!pline_level++) {
 	list_pipe_job = newjob;
@@ -1041,10 +1032,16 @@ addfd(int forked, int *save, struct multio **mfds, int fd1, int fd2, int rflag)
 	    mfds[fd1]->pipe = pipes[1 - rflag];
 	    redup(pipes[rflag], fd1);
 	    mfds[fd1]->ct = 2;
-	} else			/* add another fd to an already split stream */
+	} else {		/* add another fd to an already split stream */
+	    if(!(mfds[fd1]->ct % MULTIOUNIT)) {
+		int new = sizeof(struct multio) + sizeof(int) * mfds[fd1]->ct;
+		int old = new - sizeof(int) * MULTIOUNIT;
+		mfds[fd1] = hrealloc((char *)mfds[fd1], old, new);
+	    }
 	    mfds[fd1]->fds[mfds[fd1]->ct++] = movefd(fd2);
+	}
     }
-    if (!fdtable[subsh_close])
+    if (subsh_close >= 0 && !fdtable[subsh_close])
 	subsh_close = -1;
 }
 
@@ -1254,9 +1251,8 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 			addlinknode(args, dupstring(readnullcmd));
 		    } else
 			addlinknode(args, dupstring(nullcmd));
-		} else if (cflags & BINF_PREFIX) {
-		    zerr("command name expected", NULL, 0);
-		    errflag = lastval = 1;
+		} else if ((cflags & BINF_PREFIX) && (cflags & BINF_COMMAND)) {
+		    lastval = 0;
 		    return;
 		} else {
 		    cmdoutval = 0;
@@ -1487,12 +1483,14 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 	      "BUG: unexpanded here document\n");
 	if (fn->type == INPIPE) {
 	    if (fn->fd2 == -1) {
+		closemnodes(mfds);
 		fixfds(save);
 		execerr();
 	    }
 	    addfd(forked, save, mfds, fn->fd1, fn->fd2, 0);
 	} else if (fn->type == OUTPIPE) {
 	    if (fn->fd2 == -1) {
+		closemnodes(mfds);
 		fixfds(save);
 		execerr();
 	    }
@@ -1501,6 +1499,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 	    if (fn->type != HERESTR && xpandredir(fn, cmd->redir))
 		continue;
 	    if (errflag) {
+		closemnodes(mfds);
 		fixfds(save);
 		execerr();
 	    }
@@ -1508,6 +1507,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 	    case HERESTR:
 		fil = getherestr(fn);
 		if (fil == -1) {
+		    closemnodes(mfds);
 		    fixfds(save);
 		    if (errno != EINTR)
 			zerr("%e", NULL, errno);
@@ -1522,6 +1522,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 		else
 		    fil = open(unmeta(fn->name), O_RDWR | O_CREAT, 0666);
 		if (fil == -1) {
+		    closemnodes(mfds);
 		    fixfds(save);
 		    if (errno != EINTR)
 			zerr("%e: %s", fn->name, errno);
@@ -1547,6 +1548,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 		if (fil == -1) {
 		    char fdstr[4];
 
+		    closemnodes(mfds);
 		    fixfds(save);
 		    sprintf(fdstr, "%d", fn->fd2);
 		    zerr("%s: %e", fdstr, errno);
@@ -1568,6 +1570,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 		if (fil == -1 || dfil == -1) {
 		    if(fil != -1)
 			close(fil);
+		    closemnodes(mfds);
 		    fixfds(save);
 		    if (errno != EINTR)
 			zerr("%e: %s", fn->name, errno);
@@ -1608,7 +1611,8 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 	if (type >= CURSH) {
 	    static int (*func[]) _((Cmd)) = {
 		execcursh, exectime, execfuncdef, execfor, execwhile,
-		execrepeat, execif, execcase, execselect, execcond
+		execrepeat, execif, execcase, execselect, execcond,
+		execarith
 	    };
 
 	    if (last1 == 1)
@@ -1642,7 +1646,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 #ifdef HAVE_DEV_FD
 		int i;
 
-		for (i = 10; i < OPEN_MAX; i++)
+		for (i = 10; i <= max_zsh_fd; i++)
 		    if (fdtable[i] > 1)
 			fdtable[i]++;
 #endif
@@ -1651,7 +1655,7 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 		subsh_close = -1;
 		execshfunc(cmd, (Shfunc) hn);
 #ifdef HAVE_DEV_FD
-		for (i = 10; i < OPEN_MAX; i++)
+		for (i = 10; i <= max_zsh_fd; i++)
 		    if (fdtable[i] > 1)
 			if (--(fdtable[i]) <= 2)
 			    zclose(i);
@@ -1706,6 +1710,10 @@ execcmd(Cmd cmd, int input, int output, int how, int last1)
 			_exit(1);
 		}
 		closem(1);
+		if (coprocin)
+		    zclose(coprocin);
+		if (coprocout)
+		    zclose(coprocout);
 #ifdef HAVE_GETRLIMIT
 		if (!forked)
 		    setlimits(NULL);
@@ -1872,7 +1880,8 @@ entersubsh(int how, int cl, int fake)
 		    attachtty(jobtab[thisjob].gleader);
 	    }
 	}
-	else if (!jobtab[thisjob].gleader) {
+	else if (!jobtab[thisjob].gleader ||
+		 (setpgrp(0L, jobtab[thisjob].gleader) == -1)) {
 	    jobtab[thisjob].gleader = getpid();
 	    if (list_pipe_job != thisjob &&
 		!jobtab[list_pipe_job].gleader)
@@ -1880,8 +1889,7 @@ entersubsh(int how, int cl, int fake)
 	    setpgrp(0L, jobtab[thisjob].gleader);
 	    if (how & Z_SYNC)
 		attachtty(jobtab[thisjob].gleader);
-	} else
-	    setpgrp(0L, jobtab[thisjob].gleader);
+	}
     }
     if (!fake)
 	subsh = 1;
@@ -1917,7 +1925,7 @@ closem(int how)
 {
     int i;
 
-    for (i = 10; i < OPEN_MAX; i++)
+    for (i = 10; i <= max_zsh_fd; i++)
 	if (fdtable[i] && (!how || fdtable[i] == how))
 	    zclose(i);
 }
@@ -2349,6 +2357,21 @@ execcond(Cmd cmd)
     return !evalcond(cmd->u.cond);
 }
 
+/* evaluate a ((...)) arithmetic command */
+
+/**/
+int
+execarith(Cmd cmd)
+{
+    char *e;
+    long val = 0;
+
+    while ((e = (char *) ugetnode(cmd->args)))
+	val = matheval(e);
+    errflag = 0;
+    return !val;
+}
+
 /* perform time ... command */
 
 /**/
@@ -2407,10 +2430,18 @@ execshfunc(Cmd cmd, Shfunc shf)
 {
     List funcdef;
     char *nam;
+    LinkList last_file_list = NULL;
 
     if (errflag)
 	return;
-    deletepipejob();
+
+    if (!list_pipe) {
+	/* Without this deletejob the process table *
+	 * would be filled by a recursive function. */
+	last_file_list = jobtab[thisjob].filelist;
+	jobtab[thisjob].filelist = NULL;
+	deletejob(jobtab + thisjob);
+    }
 
     /* Are we dealing with an autoloaded shell function? */
     if (shf->flags & PM_UNDEFINED) {
@@ -2418,26 +2449,26 @@ execshfunc(Cmd cmd, Shfunc shf)
 	if (!(funcdef = getfpfunc(nam))) {
 	    zerr("function not found: %s", nam, 0);
 	    lastval = 1;
-	    return;
-	}
-	PERMALLOC {
-	    shf->flags &= ~PM_UNDEFINED;
-	    funcdef = shf->funcdef = (List) dupstruct(funcdef);
-	} LASTALLOC;
+	} else {
+	    PERMALLOC {
+		shf->flags &= ~PM_UNDEFINED;
+		funcdef = shf->funcdef = (List) dupstruct(funcdef);
+	    } LASTALLOC;
 
-	/* Execute the function definition, we just retrived */
-	doshfunc(shf->funcdef, cmd->args, shf->flags, 0);
-
-	/* See if this file defined the autoloaded function *
-	 * by name.  If so, we execute it again.            */
-	if ((shf = (Shfunc) shfunctab->getnode(shfunctab, nam))
-	    && shf->funcdef && shf->funcdef != funcdef)
+	    /* Execute the function definition, we just retrived */
 	    doshfunc(shf->funcdef, cmd->args, shf->flags, 0);
-	return;
-    }
 
-    /* Normal shell function execution */
-    doshfunc(shf->funcdef, cmd->args, shf->flags, 0);
+	    /* See if this file defined the autoloaded function *
+	     * by name.  If so, we execute it again.            */
+	    if ((shf = (Shfunc) shfunctab->getnode(shfunctab, nam))
+		&& shf->funcdef && shf->funcdef != funcdef)
+		doshfunc(shf->funcdef, cmd->args, shf->flags, 0);
+	}
+    } else
+	/* Normal shell function execution */
+	doshfunc(shf->funcdef, cmd->args, shf->flags, 0);
+    if (!list_pipe)
+	deletefilelist(last_file_list);
 }
 
 /* execute a shell function */
@@ -2577,7 +2608,10 @@ getfpfunc(char *s)
     for (; *pp; pp++) {
 	if (strlen(*pp) + strlen(s) + 1 >= PATH_MAX)
 	    continue;
-	sprintf(buf, "%s/%s", *pp, s);
+	if (**pp)
+	    sprintf(buf, "%s/%s", *pp, s);
+	else
+	    strcpy(buf, s);
 	unmetafy(buf, NULL);
 	if (!access(buf, R_OK) && (fd = open(buf, O_RDONLY)) != -1) {
 	    if ((len = lseek(fd, 0, 2)) != -1) {
@@ -2586,12 +2620,9 @@ getfpfunc(char *s)
 		if (read(fd, d, len) == len) {
 		    close(fd);
 		    d = metafy(d, len, META_REALLOC);
-		    if (useheap)
+		    HEAPALLOC {
 			r = parse_string(d);
-		    else
-			HEAPALLOC {
-			    r = parse_string(d);
-			} LASTALLOC;
+		    } LASTALLOC;
 		    zfree(d, len + 1);
 		    return r;
 		} else {
@@ -2629,7 +2660,10 @@ cancd(char *s)
 	    for (cp = cdpath; *cp; cp++) {
 		if (strlen(*cp) + strlen(s) + 1 >= PATH_MAX)
 		    continue;
-		sprintf(sbuf, "%s/%s", *cp, s);
+		if (**cp)
+		    sprintf(sbuf, "%s/%s", *cp, s);
+		else
+		    strcpy(sbuf, s);
 		if (cancd2(sbuf)) {
 		    doprintdir = -1;
 		    return dupstring(sbuf);
@@ -2670,7 +2704,6 @@ execsave(void)
     es->list_pipe_child = list_pipe_child;
     es->list_pipe_job = list_pipe_job;
     strcpy(es->list_pipe_text, list_pipe_text);
-    es->last_file_list = last_file_list;
     es->lastval = lastval;
     es->noeval = noeval;
     es->badcshglob = badcshglob;
@@ -2679,6 +2712,8 @@ execsave(void)
     es->trapreturn = trapreturn;
     es->noerrs = noerrs;
     es->subsh_close = subsh_close;
+    es->underscore = underscore;
+    underscore = ztrdup(underscore);
     es->next = exstack;
     exstack = es;
     noerrs = cmdoutpid = 0;
@@ -2698,7 +2733,6 @@ execrestore(void)
     list_pipe_child = exstack->list_pipe_child;
     list_pipe_job = exstack->list_pipe_job;
     strcpy(list_pipe_text, exstack->list_pipe_text);
-    last_file_list = exstack->last_file_list;
     lastval = exstack->lastval;
     noeval = exstack->noeval;
     badcshglob = exstack->badcshglob;
@@ -2707,6 +2741,8 @@ execrestore(void)
     trapreturn = exstack->trapreturn;
     noerrs = exstack->noerrs;
     subsh_close = exstack->subsh_close;
+    zsfree(underscore);
+    underscore = exstack->underscore;
     en = exstack->next;
     free(exstack);
     exstack = en;
