@@ -1,5 +1,5 @@
 /*
- * $Id: globals.h,v 2.40 1996/08/15 16:40:04 hzoli Exp $
+ * $Id: globals.h,v 2.44 1996/10/15 20:16:35 hzoli Exp $
  *
  * globals.h - global variables for zsh
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -200,6 +200,10 @@ EXTERN LinkList bufstack;
 
 EXTERN int inbufct;
 
+/* the flags controlling the input routines in input.c: see INP_* in zsh.h */
+
+EXTERN int inbufflags;
+
 /* != 0 if this is a subshell */
  
 EXTERN int subsh;
@@ -345,6 +349,7 @@ EXTERN pid_t mypgrp;		/* the process group of the shell */
 EXTERN char *argzero;           /* $0 */
  
 EXTERN char *hackzero;
+EXTERN char *scriptname;        /* name of script being sourced */
 
 EXTERN long lineno;             /* $LINENO       */
 EXTERN long shlvl;              /* $SHLVL        */
@@ -392,7 +397,15 @@ EXTERN int sourcelevel;
  * table is not used.  A table element is set by movefd and cleard *
  * by zclose.                                                      */
 
-EXTERN char fdtable[OPEN_MAX];
+EXTERN char *fdtable;
+
+/* The allocated size of fdtable */
+
+EXTERN int fdtable_size;
+
+/* The highest fd that marked with nonzero in fdtable */
+
+EXTERN int max_zsh_fd;
 
 /* input fd from the coprocess */
 
@@ -418,15 +431,6 @@ EXTERN FILE *shout;
 
 EXTERN FILE *bshin;
 
-/* the stack of aliases we are expanding */
- 
-EXTERN struct alias *alstack[MAXAL];
- 
-/* the alias stack pointer; also, the number of *
- * aliases currently being expanded             */
- 
-EXTERN int alstackind;
- 
 /* != 0 means we are reading input from a string */
  
 EXTERN int strin;
@@ -503,11 +507,6 @@ EXTERN int badcshglob;
  
 EXTERN int hlinesz;
  
-/* the alias expansion status - if == ALSTAT_MORE, we just finished
-        expanding an alias ending with a space */
- 
-EXTERN int alstat;
- 
 /* we have printed a 'you have stopped (running) jobs.' message */
  
 EXTERN int stopmsg;
@@ -556,6 +555,14 @@ EXTERN List sigfuncs[VSIGCOUNT];
 EXTERN int alloc_stackp;
 #endif
 
+/* Variables used by signal queueing */
+
+EXTERN int queueing_enabled;
+EXTERN sigset_t signal_mask_queue[MAX_QUEUE_SIZE];
+EXTERN int signal_queue[MAX_QUEUE_SIZE];
+EXTERN int queue_front;
+EXTERN int queue_rear;
+
 /* 1 if aliases should not be expanded */
  
 EXTERN int noaliases;
@@ -686,7 +693,7 @@ struct option optns[OPT_SIZE] = {
     {"badpattern", 		x'2', 0,    OPT_EMULATE|OPT_NONBOURNE},
     {"banghist", 		x'K', 0,    OPT_EMULATE|OPT_NONBOURNE},
     {"beep", 			x'B', 0,    OPT_ALL},
-    {"bgnice", 			'6',  0,    OPT_ALL},
+    {"bgnice", 			'6',  0,    OPT_EMULATE|OPT_NONBOURNE},
     {"braceccl", 		0,    0,    0},
     {"bsdecho", 		0,    0,    OPT_EMULATE|OPT_SH},
     {"cdablevars", 		'T',  0,    0},
