@@ -1,5 +1,5 @@
 /*
- * $Id: hist.c,v 2.25 1996/08/13 18:34:40 hzoli Exp $
+ * $Id: hist.c,v 2.27 1996/10/18 01:00:43 hzoli Exp $
  *
  * hist.c - history expansion
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -109,7 +109,7 @@ hgetc(void)
     int c = ingetc();
 
     qbang = 0;
-    if (!stophist && !alstackind) {
+    if (!stophist && !(inbufflags & INP_ALIAS)) {
 	/* If necessary, expand history characters. */
 	c = histsubchar(c);
 	if (c < 0) {
@@ -118,18 +118,18 @@ hgetc(void)
 	    return ' ';
 	}
     }
-    if (alstackind == 1 && !alstack[0] && !stophist) {
-	/* the current character c came from a history expansion        *
-	 * (alstackind == 1 && !alstack[0]) and history is not disabled *
-	 * (e.g. we are not inside single quotes). In that case, \!     *
-	 * should be treated as ! (since this \! came from a previous   *
-	 * history line where \ was used to escape the bang). So if     *
-	 * c == '\\' we fetch one more character to see if it's a bang, *
-	 * and if it is not, we unget it and reset c back to '\\'       */
+    if ((inbufflags & INP_HIST) && !stophist) {
+	/* the current character c came from a history expansion          *
+	 * (inbufflags && INP_HIST) and history is not disabled           *
+	 * (e.g. we are not inside single quotes). In that case, \!       *
+	 * should be treated as ! (since this \! came from a previous     *
+	 * history line where \ was used to escape the bang). So if       *
+	 * c == '\\' we fetch one more character to see if it's a bang,   *
+	 * and if it is not, we unget it and reset c back to '\\'         */
 	qbang = 0;
 	if (c == '\\' && !(qbang = (c = ingetc()) == bangchar))
 	    safeinungetc(c), c = '\\';
-    } else if (stophist || alstackind)
+    } else if (stophist || (inbufflags & INP_ALIAS))
 	/* If the result is a bangchar which came from history or alias  *
 	 * expansion, we treat it as an escaped bangchar, unless history *
 	 * is disabled. If stophist == 1 it only means that history is   *
@@ -471,13 +471,12 @@ histsubchar(int c)
      */
 
     lexstop = 0;
-    /* this function is called only called from hgetc and only if     *
-     * alstackind == 0. History expansion should never be done with   *
-     * nonzero alstackind (to prevent recursive history expansion and *
-     * histoty expansion of aliases). Escapes are not removed here.   *
-     * This is now handled in hgetc.                                  */
-    alstack[alstackind++] = NULL;
-    inpush(sline, INP_ALIAS); /* sline from heap, don't free */
+    /* this function is called only called from hgetc and only if      *
+     * !(inbufflags & INP_ALIAS). History expansion should never be    *
+     * done with INP_ALIAS (to prevent recursive history expansion and *
+     * histoty expansion of aliases). Escapes are not removed here.    *
+     * This is now handled in hgetc.                                   */
+    inpush(sline, INP_HIST, NULL); /* sline from heap, don't free */
     histdone |= HISTFLAG_DONE;
     if (isset(HISTVERIFY))
 	histdone |= HISTFLAG_NOEXEC | HISTFLAG_RECALL;
@@ -486,20 +485,6 @@ histsubchar(int c)
     return ingetc();
 }
 
-/* reset the alias stack for lexrestore () */
-
-/**/
-void
-clearalstack(void)
-{
-    Alias ix;
-
-    while (alstackind) {
-	ix = alstack[--alstackind];
-	ix->inuse = 0;
-    }
-}
-
 /* unget a char and remove it from chline. It can only be used *
  * to unget a character returned by hgetc.                     */
 
@@ -554,7 +539,8 @@ hungetc(int c)
 	 * (unusual means unusual for a bang like a number).  If this bang *
 	 * comes from an alias expansion, no backslash should be really    *
 	 * unget.                                                          */
-	doit = !stophist && (!alstackind || (alstackind == 1 && !alstack[0]));
+	doit = !stophist && ((inbufflags & INP_HIST) ||
+			     !(inbufflags & INP_ALIAS));
 	qbang = 0;
 	c = '\\';
 	goto escape;
@@ -604,18 +590,6 @@ hbegin(void)
     if (interact && isset(SHINSTDIN) && !strin) {
 	attachtty(mypgrp);
 	defev = curhist++;
-	if (curhist - histsiz >= 0) {
-	    struct histent *tmphist = gethistent(curhist - histsiz);
-	    if (tmphist->text) {
-		zsfree(tmphist->text);
-		tmphist->text = NULL;
-	    }
-	    if (tmphist->nwords) {
-		zfree(tmphist->words, tmphist->nwords*2*sizeof(short));
-		tmphist->words = NULL;
-		tmphist->nwords = 0;
-	    }
-	}
 	curhistent = gethistent(curhist);
 	zsfree(curhistent->text);
 	if (curhistent->nwords)
@@ -741,7 +715,7 @@ hwbegin(int offset)
      * If we're in a string, we don't need the full history line
      * and can overwrite also.
      */
-    if ((alstackind || strin) && (alstackind != 1 || alstack[0]))
+    if (((inbufflags & INP_ALIAS) || strin) && !(inbufflags & INP_HIST))
 	hwgetword = chwordpos;
     else
 	hwgetword = -1;
@@ -773,8 +747,6 @@ hwend(void)
 	    chwordpos--;
 	}
     }
-    if (alstat == ALSTAT_JUNK)
-	alstat = 0;
 }
 
 /* Go back to immediately after the last word, skipping space. */
@@ -1337,28 +1309,39 @@ resizehistents(void)
 void
 readhistfile(char *s, int err)
 {
-    char buf[1024];
+    char *buf;
     FILE *in;
     Histent ent;
     time_t tim = time(NULL);
     short *wordlist;
-    int nwordpos, nwordlist;
+    int nwordpos, nwordlist, bufsiz;
 
     if (!s)
 	return;
     if ((in = fopen(unmeta(s), "r"))) {
 	nwordlist = 16;
 	wordlist = (short *)zalloc(nwordlist*sizeof(short));
+	bufsiz = 1024;
+	buf = zalloc(bufsiz);
 
-	while (fgets(buf, sizeof(buf), in)) {
+	while (fgets(buf, bufsiz, in)) {
 	    int l = strlen(buf);
 	    char *pt, *start;
 
-	    while (l && buf[l - 1] == '\n') {
+	    while (l) {
+		while (buf[l - 1] != '\n') {
+		    buf = zrealloc(buf, 2 * bufsiz);
+		    bufsiz = 2 * bufsiz;
+		    if (!fgets(buf + l, bufsiz - l, in)) {
+			l++;
+			break;
+		    }
+		    l = strlen(buf);
+		}
 		buf[l - 1] = '\0';
 		if (l > 1 && buf[l - 2] == '\\') {
 		    buf[l - 2] = '\n';
-		    fgets(buf + l - 1, sizeof(buf) - (l - 1), in);
+		    fgets(buf + l - 1, bufsiz - (l - 1), in);
 		    l = strlen(buf);
 		} else
 		    break;
@@ -1422,6 +1405,7 @@ readhistfile(char *s, int err)
 	fclose(in);
 
 	zfree(wordlist, nwordlist*sizeof(short));
+	zfree(buf, bufsiz);
     } else if (err)
 	zerr("can't read history file", s, 0);
 }
