@@ -1,5 +1,5 @@
 /*
- * $Id: init.c,v 2.39 1996/08/13 20:36:44 hzoli Exp $
+ * $Id: init.c,v 2.46 1996/10/15 20:16:35 hzoli Exp $
  *
  * init.c - main loop and initialization routines
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -54,6 +54,9 @@ main(int argc, char **argv)
     if (*zsh_name == '-')
 	zsh_name++;
 
+    fdtable_size = OPEN_MAX;
+    fdtable = zcalloc(fdtable_size);
+
     emulate(zsh_name, 1);   /* initialises most options */
     opts[LOGINSHELL] = (**argv == '-');
     opts[MONITOR] = 1;   /* may be unset in init_io() */
@@ -110,7 +113,6 @@ loop(int toplevel)
 	    preprompt();
 	hbegin();		/* init history mech        */
 	intr();			/* interrupts on            */
-	init_aliases();		/* init alias mech          */
 	lexinit();              /* initialize lexical state */
 	if (!(list = parse_event())) {	/* if we couldn't parse a list */
 	    hend();
@@ -274,13 +276,13 @@ parseargs(char **argv)
     argzero = ztrdup(argzero);
 }
 
-static char outbuf[BUFSIZ], errbuf[BUFSIZ];
 
 /**/
 void
 init_io(void)
 {
     long ttpgrp;
+    static char outbuf[BUFSIZ], errbuf[BUFSIZ];
 
 #ifdef RSH_BUG_WORKAROUND
     int i;
@@ -314,10 +316,19 @@ init_io(void)
     }
 
     /* Make sure the tty is opened read/write. */
-    if (isatty(0))
-	SHTTY = movefd(open(ttyname(0), O_RDWR));
-    if (SHTTY == -1)
-	SHTTY = movefd(open("/dev/tty", O_RDWR));
+    if (isatty(0)) {
+	zsfree(ttystrname);
+	if ((ttystrname = ztrdup(ttyname(0))))
+	    SHTTY = movefd(open(ttystrname, O_RDWR));
+    }
+    if (SHTTY == -1 && (SHTTY = movefd(open("/dev/tty", O_RDWR))) != -1) {
+	zsfree(ttystrname);
+	ttystrname = ztrdup("/dev/tty");
+    }
+    if (SHTTY == -1) {
+	zsfree(ttystrname);
+	ttystrname = ztrdup("");
+    }
 
     /* We will only use zle if shell is interactive, *
      * SHTTY != -1, and shout != 0                   */
@@ -354,6 +365,7 @@ init_io(void)
 void
 init_shout(void)
 {
+    static char shoutbuf[BUFSIZ];
 #if defined(JOB_CONTROL) && defined(TIOCSETD) && defined(NTTYDISC)
     int ldisc = NTTYDISC;
 
@@ -363,7 +375,7 @@ init_shout(void)
     /* Associate terminal file descriptor with a FILE pointer */
     shout = fdopen(SHTTY, "w");
 #ifdef _IOFBF
-    setvbuf(shout, NULL, _IOFBF, 0);
+    setvbuf(shout, shoutbuf, _IOFBF, BUFSIZ);
 #endif
   
     gettyinfo(&shttyinfo);	/* get tty state */
@@ -416,8 +428,9 @@ init_term(void)
 	    if (!(pp = tgetstr(tccapnams[t0], &pp)))
 		tcstr[t0] = NULL, tclen[t0] = 0;
 	    else {
-		tcstr[t0] = (char *) zalloc(tclen[t0] = strlen(pp) + 1);
-		memcpy(tcstr[t0], pp, tclen[t0]);
+		tclen[t0] = strlen(pp);
+		tcstr[t0] = (char *) zalloc(tclen[t0] + 1);
+		memcpy(tcstr[t0], pp, tclen[t0] + 1);
 	    }
 	}
 
@@ -507,18 +520,14 @@ setupvals(void)
     if (opts[INTERACTIVE]) {
 	prompt  = ztrdup("%m%# ");
 	prompt2 = ztrdup("%_> ");
-	prompt3 = ztrdup("?# ");
-	prompt4 = ztrdup("+ ");
     } else {
 	prompt = ztrdup("");
 	prompt2 = ztrdup("");
-	prompt3 = ztrdup("");
-	prompt4 = ztrdup("");
     }
+    prompt3 = ztrdup("?# ");
+    prompt4 = ztrdup("+ ");
     sprompt = ztrdup("zsh: correct '%R' to '%r' [nyae]? ");
 
-    if (!(ttystrname = ztrdup(ttyname(SHTTY))))
-	ttystrname = ztrdup("");
     ifs         = ztrdup(DEFAULT_IFS);
     wordchars   = ztrdup(DEFAULT_WORDCHARS);
     postedit    = ztrdup("");
@@ -535,8 +544,12 @@ setupvals(void)
 #ifdef TIOCGWINSZ
     if (!(columns = shttyinfo.winsize.ws_col))
 	columns = 80;
+    if (columns < 2)
+	opts[USEZLE] = 0;
     if (!(lines = shttyinfo.winsize.ws_row))
 	lines = 24;
+    if (lines < 2)
+	opts[SINGLELINEZLE] = 1;
 #else
     columns = 80;
     lines = 24;
@@ -685,7 +698,7 @@ init_signals(void)
 void
 run_init_scripts(void)
 {
-    noerrexit = 1;
+    noerrexit = -1;
 
     if (emulation == EMULATE_KSH || emulation == EMULATE_SH) {
 	if (islogin)
@@ -698,7 +711,7 @@ run_init_scripts(void)
 	    if (s && !parsestr(s)) {
 		singsub(&s);
 		noerrs = 0;
-		sourcehome(s);
+		source(s);
 	    }
 	    noerrs = 0;
 	} else
@@ -767,6 +780,7 @@ source(char *s)
     int tempfd, fd, cj, oldlineno;
     int oldshst, osubsh, oloops;
     FILE *obshin;
+    char *old_scriptname = scriptname;
 
     if (!s || (tempfd = movefd(open(unmeta(s), O_RDONLY))) == -1) {
 	return 1;
@@ -787,6 +801,7 @@ source(char *s)
     lineno = 0;
     loops  = 0;
     dosetopt(SHINSTDIN, 0, 1);
+    scriptname = s;
 
     sourcelevel++;
     loop(0);			/* loop through the file to be sourced        */
@@ -804,6 +819,8 @@ source(char *s)
     dosetopt(SHINSTDIN, oldshst, 1); /* SHINSTDIN option                     */
     errflag = 0;
     retflag = 0;
+    scriptname = old_scriptname;
+
     return 0;
 }
 
@@ -827,15 +844,6 @@ sourcehome(char *s)
     source(buf);
 }
 
-/**/
-void
-init_aliases(void)
-{
-    alstackind = 0;		/* reset alias stack */
-    alstat = 0;
-    isfirstln = 1;
-}
-
 /**/
 void
 compctlsetup(void)
