@@ -1,5 +1,5 @@
 /*
- * $Id: input.c,v 2.9 1996/08/11 19:46:50 hzoli Exp $
+ * $Id: input.c,v 2.11 1996/10/15 20:16:35 hzoli Exp $
  *
  * input.c - read and store lines of input
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -35,17 +35,20 @@
  * history expansion code a character at a time.  Input is stored on a
  * stack, which allows insertion of strings into the input, possibly with
  * flags marking the end of alias expansion, with minimal copying of
- * strings.
+ * strings.  The same stack is used to record the fact that the input
+ * is a history or alias expansion and to store the alias while it is in use.
  * 
  * Input is taken either from zle, if appropriate, or read directly from
  * the input file, or may be supplied by some other part of the shell (such
  * as `eval' or $(...) substitution).  In the last case, it should be
  * supplied by pushing a new level onto the stack, via inpush(input_string,
- * flag); if the current input really needs to be altered, use
+ * flag, alias); if the current input really needs to be altered, use
  * inputsetline(input_string, flag).  `Flag' can include or's of INP_FREE
  * (if the input string is to be freed when used), INP_CONT (if the input
- * is to continue onto what's already in the input queue), or INP_ALIAS
- * (used as a mark to pop the alias stack).
+ * is to continue onto what's already in the input queue), INP_ALIAS
+ * (push supplied alias onto stack) or INP_HIST (ditto, but used to
+ * mark history expansion).  `alias' is ignored unless INP_ALIAS or
+ * INP_HIST is supplied.  INP_ALIAS is always set if INP_HIST is.
  * 
  * Note that the input string is itself used as the input buffer: it is not
  * copied, nor is it every written back to, so using a constant string
@@ -53,19 +56,24 @@
  * it is necessary that that heap last as long as the operation of reading
  * the string.  After the string is read, the stack should be popped with
  * inpop(), which effectively flushes any unread input as well as restoring
- * the previous input state,
+ * the previous input state.
  *
- * PWS 1995/06/30
+ * The flags INP_ALMORE is set when a popped alias ends in a space so that
+ * the next word should also be alias expanded.  The flag is reset in
+ * exalias().  The internal flag INP_ALCONT shows that the stack element
+ * was pushed by an alias expansion and should not be needed elsewhere.
+ *
+ * PWS 1996/12/10
  */
 
 #include "zsh.h"
 
-/* Input buffer variables:  only inbufct is global, see globals.h. */
+/* Input buffer variables:  inbufct and inbufflags are in globals.h. */
 static char *inbuf;		/* Current input buffer */
 static char *inbufptr;		/* Pointer into input buffer */
+static char *inbufpush;		/* Character at which to re-push alias */
 static int inbufleft;		/* Characters left in current input
 				   stack element */
-static int inbufflags;		/* flags: see INP_* in zsh.h  */
 
 static int lastc;		/* used as flag that end of line was reached */
 
@@ -76,6 +84,7 @@ static int lastc;		/* used as flag that end of line was reached */
 
 struct instacks {
     char *buf, *bufptr;
+    Alias alias;
     int bufleft, bufct, flags;
 };
 static struct instacks *instack, *instacktop;
@@ -91,29 +100,6 @@ static struct instacks *instack, *instacktop;
 
 static int instacksz = INSTACK_INITIAL;
 
-/*
- * Stack to deal with popped aliases.
- * The problem is that we reach the end of the alias text and pop
- * the alias at the point where we read the next character
- * afterwards, which is probably whitespace, *then* go back and
- * look to see if the expanded text needs re-expanding.  This
- * means we need to keep track of the aliases we've popped and
- * maybe add them back in if we unget the whitespace character.
- *
- * What happens is that ingetc(), when it pops an alias, sticks it on the
- * stack, and inungetc() looks at that to see if it has to restore the
- * alias(es); if so, the commands to pop an alias go back in the input
- * queue. If ingetc() is called again, it sets the stack to zero.
- * 
- * We also need to be careful about `lexstop' when reading from a string.
- * 
- * Historical note:  the code used to add a bogus space to the end of
- * aliases, after which the alias would be popped.  With the new
- * history code, which keeps track of spaces correctly, this won't work.
- */
-static Alias *inalstack, *inalstacktop;
-static int inalstacksz = INSTACK_INITIAL;
-
 /* Read a line from bshin.  Convert tokens and   *
  * null characters to Meta c^32 character pairs. */
 
@@ -165,7 +151,8 @@ shingetline(void)
 int
 ingetc(void)
 {
-    inalstacktop = inalstack;
+    if (lexstop)
+	return lastc = ' ';
     for (;;) {
 	if (inbufleft) {
 	    inbufleft--;
@@ -174,37 +161,11 @@ ingetc(void)
 		continue;
 	    return lastc;
 	}
-	/*
-	 * No characters in input buffer.
-	 * See if we can pop the alias stack at this point.
-	 */
-	if ((inbufflags & INP_ALIAS) && alstackind > 0) {
-	    /*
-	     * Flag that we should pop the alias stack at this point.
-	     * Either we have reached the end of an alias expansion,
-	     * or the end of a history expansion.
-	     */
-	    Alias ix;
-	    char *t;
-	    
-	    ix = alstack[--alstackind];
-	    if (ix) {
-		/* a real alias:  mark it as unused. */
-		ix->inuse = 0;
-		t = ix->text;
-		if (*t && t[strlen(t) - 1] == ' ') {
-		    alstat = ALSTAT_MORE;
-		    histbackword();
-		} else
-		    alstat = ALSTAT_JUNK;
-		inalpush(ix);
-	    }
-	}
 
 	/* If the next element down the input stack is a continuation of
 	 * this, use it.
 	 */ 
-	if (inbufflags & (INP_CONT|INP_ALIAS)) {
+	if (inbufflags & INP_CONT) {
 	    inpoptop();
 	    continue;
 	}
@@ -324,7 +285,7 @@ inputsetline(char *str, int flags)
      * of whether the input stack continues, and whether there
      * is an extra space to add on at the end.
      */
-    if (flags & (INP_ALIAS|INP_CONT))
+    if (flags & INP_CONT)
 	inbufct += inbufleft;
     else
 	inbufct = inbufleft;
@@ -359,7 +320,7 @@ inungetc(int c)
 	    inbufleft++;
 	}
 #ifdef DEBUG
-        else if (!(inbufflags & (INP_ALIAS|INP_CONT))) {
+        else if (!(inbufflags & INP_CONT)) {
 	    /* Just for debugging */
 	    fprintf(stderr, "Attempt to inungetc() at start of input.\n");
 	}
@@ -373,11 +334,27 @@ inungetc(int c)
 	     */
 	    char *cback = (char *)zcalloc(2);
 	    cback[0] = (char) c;
-	    inpush(cback, INP_FREE|INP_CONT);
+	    inpush(cback, INP_FREE|INP_CONT, NULL);
+	}
+	/* If we are back at the start of a segment,
+	 * we may need to restore an alias popped from the stack.
+	 * Note this may be a dummy (history expansion) entry.
+	 */
+	if (inbufptr == inbufpush && inbufflags & INP_ALCONT) {
+	    /*
+	     * Go back up the stack over all entries which were alias
+	     * expansions and were pushed with nothing remaining to read.
+	     */
+	    do {
+		if (instacktop->alias)
+		    instacktop->alias->inuse = 1;
+		instacktop++;
+	    } while ((instacktop->flags & INP_ALCONT) && !instacktop->bufleft);
+	    inbufflags = INP_CONT|INP_ALIAS;
+	    inbufleft = 0;
+	    inbuf = inbufptr = "";
 	}
     }
-    if (inalstacktop > inalstack)
-	inalrestore();
 }
 
 /* stuff a whole file into the input queue and print it */
@@ -406,7 +383,8 @@ stuff(char *fn)
     }
     fclose(in);
     buf[len] = '\0';
-    fwrite(buf, len, 1, stdout);
+    fwrite(buf, len, 1, stderr);
+    fflush(stderr);
     inputsetline(metafy(buf, len, META_REALLOC), INP_FREE);
     return 0;
 }
@@ -430,13 +408,41 @@ inerrflush(void)
 
 /**/
 void
-inpush(char *str, int flags)
+inpush(char *str, int flags, Alias inalias)
 {
     if (!instack) {
 	/* Initial stack allocation */
 	instack = (struct instacks *)zalloc(instacksz*sizeof(struct instacks));
 	instacktop = instack;
-    } else if (instacktop == instack + instacksz) {
+    }
+
+    instacktop->buf = inbuf;
+    instacktop->bufptr = inbufptr;
+    instacktop->bufleft = inbufleft;
+    instacktop->bufct = inbufct;
+    inbufflags &= ~(INP_ALCONT|INP_ALMORE);
+    if (flags & (INP_ALIAS|INP_HIST)) {
+	/*
+	 * Text is expansion for history or alias, so continue
+	 * back to old level when done.  Also mark stack top
+	 * as alias continuation so as to back up if necessary,
+	 * and mark alias as in use.
+	 */
+	flags |= INP_CONT|INP_ALIAS;
+	instacktop->flags = inbufflags | INP_ALCONT;
+	if ((instacktop->alias = inalias))
+	    inalias->inuse = 1;
+    } else {
+	/* If we are continuing an alias expansion, record the alias
+	 * expansion in new set of flags (do we need this?)
+	 */
+	if (((instacktop->flags = inbufflags) & INP_ALIAS) &&
+	    (flags & INP_CONT))
+	    flags |= INP_ALIAS;
+    }
+
+    instacktop++;
+    if (instacktop == instack + instacksz) {
 	/* Expand the stack */
 	instack = (struct instacks *)
 	    realloc(instack,
@@ -444,15 +450,13 @@ inpush(char *str, int flags)
 	instacktop = instack + instacksz;
 	instacksz += INSTACK_EXPAND;
     }
-    instacktop->buf = inbuf;
-    instacktop->bufptr = inbufptr;
-    instacktop->bufleft = inbufleft;
-    instacktop->bufct = inbufct;
-    instacktop->flags = inbufflags;
+    /*
+     * We maintain the entry above the highest one with real
+     * text as a flag to inungetc() that it can stop re-pushing the stack.
+     */
+    instacktop->flags = 0;
 
-    instacktop++;
-
-    inbuf = 0;
+    inbufpush = inbuf = NULL;
 
     inputsetline(str, flags);
 }
@@ -465,14 +469,27 @@ inpoptop(void)
 {
     if (inbuf && (inbufflags & INP_FREE))
 	free(inbuf);
-	
+
     instacktop--;
 
     inbuf = instacktop->buf;
-    inbufptr = instacktop->bufptr;
+    inbufptr = inbufpush = instacktop->bufptr;
     inbufleft = instacktop->bufleft;
     inbufct = instacktop->bufct;
     inbufflags = instacktop->flags;
+
+    if (!(inbufflags & INP_ALCONT))
+	return;
+
+    if (instacktop->alias) {
+	char *t = instacktop->alias->text;
+	/* a real alias:  mark it as unused. */
+	instacktop->alias->inuse = 0;
+	if (*t && t[strlen(t) - 1] == ' ') {
+	    inbufflags |= INP_ALMORE;
+	    histbackword();
+	}
+    }
 }
 
 /* Remove the top element of the stack and all its continuations. */
@@ -484,36 +501,8 @@ inpop(void)
     int remcont;
 
     do {
-	remcont = inbufflags & (INP_CONT|INP_ALIAS);
+	remcont = inbufflags & INP_CONT;
 
 	inpoptop();
     } while (remcont);
 }
-
-/**/
-void
-inalpush(Alias al)
-{
-    if (!inalstack) {
-	 inalstack = (Alias *)zalloc(inalstacksz*sizeof(Alias));
-	 inalstacktop = inalstack;
-    } else if (inalstacktop - inalstack == inalstacksz) {
-	inalstack = (Alias *)
-	    realloc(inalstack, (inalstacksz + INSTACK_EXPAND)*sizeof(Alias));
-	inalstacktop = inalstack + inalstacksz;
-	inalstacksz += INSTACK_EXPAND;
-    }
-    *inalstacktop++ = al;
-}
-
-/**/
-void
-inalrestore(void)
-{
-    while (inalstacktop > inalstack) {
-	Alias al = *--inalstacktop;
-	alstack[alstackind++] = al;
-	al->inuse = 1;
-	inpush("", INP_ALIAS);
-    }
-}
