@@ -1,5 +1,5 @@
 /*
- * $Id: loop.c,v 2.2 1996/07/29 02:16:42 hzoli Exp $
+ * $Id: loop.c,v 2.4 1996/10/15 20:16:35 hzoli Exp $
  *
  * loop.c - loop execution
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -250,7 +250,8 @@ execif(Cmd cmd)
     i = node->ifls;
     t = node->thenls;
 
-    noerrexit = 1;
+    if (!noerrexit)
+	noerrexit = 1;
     while (*i) {
 	execlist(*i, 1, 0);
 	if (!lastval)
