@@ -1,5 +1,5 @@
 /*
- * $Id: parse.c,v 2.25 1996/08/14 16:48:19 hzoli Exp $
+ * $Id: parse.c,v 2.28 1996/10/15 20:16:35 hzoli Exp $
  *
  * parse.c - parser
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -605,7 +605,7 @@ par_if(Cmd c)
     unsigned char nc;
     LinkList ifsl, thensl;
     LinkNode no;
-    int ni = 0, nt = 0;
+    int ni = 0, nt = 0, usebrace = 0;
     List l, *ll;
 
     ifsl = newlinklist();
@@ -634,6 +634,7 @@ par_if(Cmd c)
 	xtok = FI;
 	nc = cmdstack[cmdsp - 1] == CS_IF ? CS_IFTHEN : CS_ELIFTHEN;
 	if (tok == THEN) {
+	    usebrace = 0;
 	    cmdpop();
 	    cmdpush(nc);
 	    yylex();
@@ -643,6 +644,7 @@ par_if(Cmd c)
 	    cmdpop();
 	} else {
 	    if (tok == INBRACE) {
+		usebrace = 1;
 		cmdpop();
 		cmdpush(nc);
 		yylex();
@@ -676,10 +678,19 @@ par_if(Cmd c)
 	cmdpush(CS_ELSE);
 	while (tok == SEPER)
 	    yylex();
-	l = par_list();
-	if (tok != FI) {
-	    cmdpop();
-	    YYERRORV;
+	if (tok == INBRACE && usebrace) {
+	    yylex();
+	    l = par_list();
+	    if (tok != OUTBRACE) {
+		cmdpop();
+		YYERRORV;
+	    }
+	} else {
+	    l = par_list();
+	    if (tok != FI) {
+		cmdpop();
+		YYERRORV;
+	    }
 	}
 	addlinknode(thensl, l);
 	nt++;
@@ -881,18 +892,12 @@ par_dinbrack(Cmd c)
 void
 par_dinpar(Cmd c)
 {
-    c->type = SIMPLE;
-    addlinknode(c->args, dupstring("builtin"));
-    addlinknode(c->args, dupstring("let"));
+    c->type = CARITH;
     incmdpos = 0;
     yylex();
     if (tok != DOUTPAR)
 	YYERRORV;
     addlinknode(c->args, tokstr);
-    if (underscore)
-	free(underscore);
-    underscore = ztrdup(tokstr);
-    untokenize(underscore);
     incmdpos = 1;
     yylex();
 }
@@ -1109,7 +1114,7 @@ par_cond_2(void)
 	    YYERROR;
     s1 = tokstr;
     if (condlex == testlex)
-	dble = (*s1 == '-' && strspn(s1+1, "abcdefghknoprstuwxzLOGS") == 1
+	dble = (*s1 == '-' && strspn(s1+1, "abcdefghknoprstuwxzLONGS") == 1
 		  && !s1[2]);
     condlex();
     if (tok == INANG || tok == OUTANG) {
