@@ -1,5 +1,5 @@
 /*
- * $Id: signals.c,v 2.21 1996/08/12 01:36:46 hzoli Exp $
+ * $Id: signals.c,v 2.24 1996/10/15 20:16:35 hzoli Exp $
  *
  * signals.c - signals handling code
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -343,55 +343,6 @@ signal_suspend(int sig, int sig2)
     return ret;
 }
 
-
-/* Use a circular queue to save signals caught during    *
- * critical sections of code.  You call queue_signals to *
- * start queueing, and unqueue_signals to process the    *
- * queue and stop queueing.  Since the kernel doesn't    *
- * queue signals, it is probably overkill for zsh to do  *
- * this, but it shouldn't hurt anything to do it anyway. */
-
-/* Right now I'm queueing all signals, but maybe we only *
- * need to queue SIGCHLD.  Anybody know?                 */
-
-
-#define MAX_QUEUE_SIZE 16
-
-static int queueing_enabled;
-static sigset_t signal_mask_queue[MAX_QUEUE_SIZE];
-static int signal_queue[MAX_QUEUE_SIZE];
-static int queue_front = 0;
-static int queue_rear = 0;
-
-/* Start queueing up any received signals rather *
- * than handling them.                           */
-
-/**/
-void
-queue_signals(void)
-{
-    queueing_enabled++;        /* signals are now being queued */
-}
-
-/* Process all queued signals */
-
-/**/
-void
-unqueue_signals(void)
-{
-    DPUTS(!queueing_enabled, "BUG: unqueue_signals called but not queueing");
-    if (--queueing_enabled)
-	return;
-
-    while (queue_front != queue_rear) {            /* while signals in queue */
-	sigset_t oset;
-        queue_front = ++queue_front % MAX_QUEUE_SIZE;
-        oset = signal_setmask(signal_mask_queue[queue_front]);
-        handler(signal_queue[queue_front]);        /* handle queued signal   */
-	signal_setmask(oset);
-    }
-}
- 
 /* What flavor of waitpid/wait3/wait shall we use? */
  
 #ifdef HAVE_WAITPID
@@ -526,6 +477,8 @@ handler(int sig)
         else {
             extern int list_pipe, simple_pline;
  
+	    if (isset(PRIVILEGED) && isset(INTERACTIVE) && noerrexit < 0)
+		zexit(SIGINT, 1);
             if (list_pipe || chline || simple_pline) {
                 breaks = loops;
                 errflag = 1;
