@@ -1,5 +1,5 @@
 /*
- * $Id: signals.h,v 2.2 1996/06/28 02:05:24 hzoli Exp $
+ * $Id: signals.h,v 2.4 1996/10/15 20:16:35 hzoli Exp $
  *
  * signals.h - header file for signals handling code
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -70,3 +70,29 @@
 /* return a signal to it default action */
 #define signal_default(S)  signal(S, SIG_DFL)
 
+/* Use a circular queue to save signals caught during    *
+ * critical sections of code.  You call queue_signals to *
+ * start queueing, and unqueue_signals to process the    *
+ * queue and stop queueing.  Since the kernel doesn't    *
+ * queue signals, it is probably overkill for zsh to do  *
+ * this, but it shouldn't hurt anything to do it anyway. */
+
+/* Right now I'm queueing all signals, but maybe we only *
+ * need to queue SIGCHLD.  Anybody know?                 */
+
+#define MAX_QUEUE_SIZE 16
+
+#define queue_signals()    (queueing_enabled++)
+
+#define unqueue_signals()  do { \
+    DPUTS(!queueing_enabled, "BUG: unqueue_signals called but not queueing"); \
+    if (!--queueing_enabled) { \
+	while (queue_front != queue_rear) {      /* while signals in queue */ \
+	    sigset_t oset; \
+	    queue_front = ++queue_front % MAX_QUEUE_SIZE; \
+	    oset = signal_setmask(signal_mask_queue[queue_front]); \
+	    handler(signal_queue[queue_front]);  /* handle queued signal   */ \
+	    signal_setmask(oset); \
+	} \
+    } \
+} while (0)
