@@ -1,5 +1,5 @@
 /*
- * $Id: text.c,v 2.5 1996/06/28 02:05:24 hzoli Exp $
+ * $Id: text.c,v 2.6 1996/10/15 20:16:35 hzoli Exp $
  *
  * text.c - textual representations of syntax trees
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -272,6 +272,11 @@ gettext2(struct node *n)
 	    gt2(nn->u.cond);
 	    taddstr(" ]]");
 	    break;
+	case CARITH:
+	    taddstr("((");
+	    taddlist(nn->args);
+	    taddstr("))");
+	    break;
 	case CREPEAT:
 	    taddstr("repeat ");
 	    taddlist(nn->args);
