@@ -1,5 +1,5 @@
 /*
- * $Id: utils.c,v 2.51 1996/08/14 16:21:47 hzoli Exp $
+ * $Id: utils.c,v 2.61 1996/10/15 20:16:35 hzoli Exp $
  *
  * utils.c - miscellaneous utilities
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -52,7 +52,13 @@ zerr(char *fmt, char *str, int num)
 	return;
     errflag = 1;
     trashzle();
-    nicezputs(isset(SHINSTDIN) ? "zsh" : argzero, stderr);
+    /*
+     * scriptname is set when sourcing scripts, so that we get the
+     * correct name instead of the generic name of whatever
+     * program/script is running.
+     */
+    nicezputs(isset(SHINSTDIN) ? "zsh" :
+	      scriptname ? scriptname : argzero, stderr);
     fputs(": ", stderr);
     zerrnam(NULL, fmt, str, num);
 }
@@ -67,7 +73,7 @@ zerrnam(char *cmd, char *fmt, char *str, int num)
 	errflag = 1;
 	trashzle();
 	if(unset(SHINSTDIN)) {
-	    nicezputs(argzero, stderr);
+	    nicezputs(scriptname ? scriptname : argzero, stderr);
 	    fputs(": ", stderr);
 	}
 	nicezputs(cmd, stderr);
@@ -652,22 +658,25 @@ checkmailpath(char **s)
 	    LinkList l;
 	    DIR *lock = opendir(unmeta(*s));
 	    char buf[PATH_MAX * 2], **arr, **ap;
-	    struct dirent *de;
 	    int ct = 1;
 
 	    if (lock) {
+		char *fn;
 		HEAPALLOC {
 		    pushheap();
 		    l = newlinklist();
-		    readdir(lock);
-		    readdir(lock);
-		    while ((de = zreaddir(lock))) {
+		    while ((fn = zreaddir(lock))) {
 			if (errflag)
 			    break;
+			/* Ignore `.' and `..'. */
+			if (fn[0] == '.' &&
+			    (fn[1] == '\0' ||
+			     (fn[1] == '.' && fn[2] == '\0')))
+			    continue;
 			if (u)
-			    sprintf(buf, "%s/%s?%s", *s, de->d_name, u);
+			    sprintf(buf, "%s/%s?%s", *s, fn, u);
 			else
-			    sprintf(buf, "%s/%s", *s, de->d_name);
+			    sprintf(buf, "%s/%s", *s, fn);
 			addlinknode(l, dupstring(buf));
 			ct++;
 		    }
@@ -844,6 +853,10 @@ adjustwinsize(void)
     if (shttyinfo.winsize.ws_row)
 	lines = shttyinfo.winsize.ws_row;
     if (oldcols != columns) {
+	if (columns < 2)
+	    opts[USEZLE] = 0;
+	if (lines < 2)
+	    opts[SINGLELINEZLE] = 1;
 	if (zleactive) {
 	    resetneeded = winchanged = 1;
 	    refresh();
@@ -868,12 +881,17 @@ movefd(int fd)
 #else
 	int fe = movefd(dup(fd));
 #endif
-	close(fd);
-	fdtable[fd] = 0;
+	zclose(fd);
 	fd = fe;
     }
-    if(fd != -1)
+    if(fd != -1) {
+	if (fd > max_zsh_fd) {
+	    while (fd >= fdtable_size)
+		fdtable = zrealloc(fdtable, (fdtable_size *= 2));
+	    max_zsh_fd = fd;
+	}
 	fdtable[fd] = 1;
+    }
     return fd;
 }
 
@@ -883,14 +901,15 @@ movefd(int fd)
 void
 redup(int x, int y)
 {
-    if(x < 0) {
-	close(y);
-	fdtable[y] = 0;
-    } else if (x != y) {
+    if(x < 0)
+	zclose(y);
+    else if (x != y) {
+	while (y >= fdtable_size)
+	    fdtable = zrealloc(fdtable, (fdtable_size *= 2));
 	dup2(x, y);
-	fdtable[y] = fdtable[x];
-	close(x);
-	fdtable[x] = 0;
+	if ((fdtable[y] = fdtable[x]) && y > max_zsh_fd)
+	    max_zsh_fd = y;
+	zclose(x);
     }
 }
 
@@ -900,8 +919,15 @@ redup(int x, int y)
 int
 zclose(int fd)
 {
-    if (fd >= 0)
+    if (fd >= 0) {
 	fdtable[fd] = 0;
+	while (max_zsh_fd > 0 && !fdtable[max_zsh_fd])
+	    max_zsh_fd--;
+	if (fd == coprocin)
+	    coprocin = -1;
+	if (fd == coprocout)
+	    coprocout = -1;
+    }
     return close(fd);
 }
 
@@ -1035,10 +1061,16 @@ long
 zstrtol(const char *s, char **t, int base)
 {
     long ret = 0;
+    int neg;
 
     while (inblank(*s))
 	s++;
 
+    if ((neg = (*s == '-')))
+	s++;
+    else if (*s == '+')
+	s++;
+
     if (!base)
 	if (*s != '0')
 	    base = 10;
@@ -1056,7 +1088,7 @@ zstrtol(const char *s, char **t, int base)
 	    ret = ret * base + (idigit(*s) ? (*s - '0') : (*s & 0x1f) + 9);
     if (t)
 	*t = (char *)s;
-    return ret;
+    return neg ? -ret : ret;
 }
 
 /* Convert string to quad_t. */
@@ -1118,7 +1150,7 @@ getquery(char *valid_chars)
     int isem = !strcmp(term, "emacs");
 
 #ifdef FIONREAD
-    int val;
+    int val = 0;
 #endif
 
     attachtty(mypgrp);
@@ -1150,6 +1182,8 @@ getquery(char *valid_chars)
 	    break;
 	}
 	feep();
+	if (icntrl(c))
+	    write(2, "\b \b", 3);
 	write(2, "\b \b", 3);
     }
     if (isem) {
@@ -2548,6 +2582,7 @@ spname(char *oldname)
     char *p, spnameguess[PATH_MAX + 1], spnamebest[PATH_MAX + 1];
     static char newname[PATH_MAX + 1];
     char *new = newname, *old;
+    int bestdist = 200, thisdist;
 
     old = oldname;
     for (;;) {
@@ -2561,8 +2596,15 @@ spname(char *oldname)
 	    if (p < spnameguess + PATH_MAX)
 		*p++ = *old;
 	*p = '\0';
-	if (mindist(newname, spnameguess, spnamebest) >= 3)
-	    return NULL;
+	if ((thisdist = mindist(newname, spnameguess, spnamebest)) >= 3) {
+	    if (bestdist < 3) {
+		strcpy(new, spnameguess);
+		strcat(new, old);
+		return newname;
+	    } else
+	    	return NULL;
+	} else
+	    bestdist = thisdist;
 	for (p = spnamebest; (*new = *p++);)
 	    new++;
     }
@@ -2574,7 +2616,7 @@ mindist(char *dir, char *mindistguess, char *mindistbest)
 {
     int mindistd, nd;
     DIR *dd;
-    struct dirent *de;
+    char *fn;
     char buf[PATH_MAX];
 
     if (dir[0] == '\0')
@@ -2587,11 +2629,11 @@ mindist(char *dir, char *mindistguess, char *mindistbest)
     }
     if (!(dd = opendir(unmeta(dir))))
 	return mindistd;
-    while ((de = zreaddir(dd))) {
-	nd = spdist(de->d_name, mindistguess,
+    while ((fn = zreaddir(dd))) {
+	nd = spdist(fn, mindistguess,
 		    (int)strlen(mindistguess) / 4 + 1);
 	if (nd <= mindistd) {
-	    strcpy(mindistbest, de->d_name);
+	    strcpy(mindistbest, fn);
 	    mindistd = nd;
 	    if (mindistd == 0)
 		break;
@@ -2912,13 +2954,13 @@ metafy(char *buf, int len, int heap)
 	    break;
 #endif
 	}
-	e = p = buf + len;
+	p = buf + len;
+	e = t = buf + len + meta;
 	while (meta) {
-	    if (imeta(*--p)) {
-		*p ^= 32;
+	    if (imeta(*--t = *--p)) {
+		*t-- ^= 32;
+		*t = Meta;
 		meta--;
-		for (t = e++; t > p; t[0] = t[-1], --t);
-		*p = Meta;
 	    }
 	}
     }
@@ -2930,15 +2972,14 @@ metafy(char *buf, int len, int heap)
 char *
 unmetafy(char *s, int *len)
 {
-    char *p;
-    for (p = s; *p; p++) {
-	if (*p == Meta) {
-	    chuck(p);
-	    *p ^= 32;
-	}
-    }
+    char *p, *t;
+
+    for (p = s; *p && *p != Meta; p++);
+    for (t = p; (*t = *p++);)
+	if (*t++ == Meta)
+	    t[-1] = *p++ ^ 32;
     if (len)
-	*len = p - s;
+	*len = t - s;
     return s;
 }
 
@@ -3086,15 +3127,12 @@ ztrsub(char const *t, char const *s)
 }
 
 /**/
-struct dirent *
+char *
 zreaddir(DIR *dir)
 {
-    struct dirent *de;
+    struct dirent *de = readdir(dir);
 
-    if ((de = readdir(dir)))
-	metafy(de->d_name, -1, META_STATIC);
-
-    return de;
+    return de ? metafy(de->d_name, -1, META_STATIC) : NULL;
 }
 
 /* Unmetafy and output a string. */
