@@ -1,5 +1,5 @@
 /*
- * $Id: zle_main.c,v 2.26 1996/08/12 01:39:05 hzoli Exp $
+ * $Id: zle_main.c,v 2.29 1996/10/25 19:27:45 hzoli Exp $
  *
  * zle_main.c - main routines for line editor
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -277,7 +277,7 @@ getkey(int keytmout)
 #  else
 	    ioctl(SHTTY, TCSETA, &shttyinfo.tio);
 #  endif
-	    return (r <= 0) ? -1 : cc;
+	    return (r <= 0) ? EOF : cc;
 # endif
 #endif
 	}
@@ -419,7 +419,6 @@ zleread(char *lp, char *rp)
 	    putc('\r', shout);
 	if (tmout)
 	    alarm(tmout);
-	genprompts();
 	zleactive = 1;
 	resetneeded = 1;
 	refresh();
@@ -991,7 +990,15 @@ bin_bindkey(char *name, char **argv, char *ops, int junc)
 			}
 			(ky = (Key) keybindtab->getnode(keybindtab, s))->prefixct--;
 			if (!ky->prefixct) {
+			    int *otab = ops['a'] ? mainbindtab : altbindtab;
 			    tab[STOUC(*s)] = ky->func;
+			    /*
+			     * If the bindtab we are not using also
+			     * adds this key as a prefix, it must also
+			     * be reset.
+			     */
+			    if (otab[STOUC(*s)] == z_prefix)
+				otab[STOUC(*s)] = ky->func;
 			    if (ky->func != z_sendstring)
 				free(keybindtab->removenode(keybindtab, s));
 			}
