@@ -1,5 +1,5 @@
 /*
- * $Id: zle_misc.c,v 2.27 1996/08/13 20:36:44 hzoli Exp $
+ * $Id: zle_misc.c,v 2.30 1996/10/15 20:16:35 hzoli Exp $
  *
  * zle_misc.c - miscellaneous editor routines
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -835,8 +835,8 @@ putprompt(char *fmin, int *lenp, int *wp, int cnt)
     if (wp) {
 	*wp = bp - bl0 - lensb;
 	if (pmpt != rpmpt) {
-	    *wp %= COLUMNS;
-	    if (*wp == COLUMNS - 1) {
+	    *wp %= columns;
+	    if (*wp == columns - 1) {
 		addbufspc(1);
 		*wp = 0;
 		*bp++ = ' ';
@@ -1272,7 +1272,7 @@ putpromptchar(int doprint, int endchar)
 	    }
 	} else if (doprint) {
 	    addbufspc(1);
-	    pputc(*fm);
+	    pputc(*fm == Meta ? *++fm ^ 32 : *fm);
 	}
     }
 
