@@ -1,5 +1,5 @@
 /*
- * $Id: zsh.h,v 2.44 1996/08/15 16:41:25 hzoli Exp $
+ * $Id: zsh.h,v 2.49 1996/10/15 20:16:35 hzoli Exp $
  *
  * zsh.h - standard header file
  *
@@ -10,9 +10,9 @@
  *
  * Permission is hereby granted, without written agreement and without
  * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that the
- * above copyright notice and the following two paragraphs appear in
- * all copies of this software.
+ * software and to distribute modified versions of this software for any
+ * purpose, provided that the above copyright notice and the following
+ * two paragraphs appear in all copies of this software.
  *
  * In no event shall Paul Falstad or the Zsh Development Group be liable
  * to any party for direct, indirect, special, incidental, or consequential
@@ -26,6 +26,7 @@
  * provided hereunder is on an "as is" basis, and Paul Falstad and the
  * Zsh Development Group have no obligation to provide maintenance,
  * support, updates, enhancements, or modifications.
+ *
  */
 
 /* We use <config.h> instead of "config.h" so that a compilation        *
@@ -193,8 +194,11 @@ enum {
 
 /* Flags for input stack */
 #define INP_FREE      (1<<0)	/* current buffer can be free'd            */
-#define INP_ALIAS     (1<<1)	/* alias or history mark at end of word    */
-#define INP_CONT      (1<<2)	/* continue onto previously stacked input  */
+#define INP_ALIAS     (1<<1)	/* expanding alias or history              */
+#define INP_HIST      (1<<2)	/* expanding history                       */
+#define INP_CONT      (1<<3)	/* continue onto previously stacked input  */
+#define INP_ALMORE    (1<<4)	/* alias ended with space, continue expn.  */
+#define INP_ALCONT    (1<<5)	/* stack is continued from alias expn.     */
 
 /* Flags for metafy */
 #define META_REALLOC	0
@@ -237,7 +241,7 @@ typedef struct comp      *Comp;
 typedef struct redir     *Redir;
 typedef struct complist  *Complist;
 typedef struct heap      *Heap;
-typedef struct heappos   *Heappos;
+typedef struct heapstack *Heapstack;
 typedef struct histent   *Histent;
 typedef struct compctlp  *Compctlp;
 typedef struct compctl   *Compctl;
@@ -403,6 +407,7 @@ struct cmd {
 #define CCASE    9
 #define CSELECT 10
 #define COND    11
+#define CARITH  12
 
 /* flags for command modifiers */
 #define CFLAG_EXEC	(1<<0)	/* exec ...    */
@@ -478,9 +483,9 @@ struct whilecmd {
     List loop;			/* list to execute until condition met */
 };
 
-/* The maximum number of redirections you *
- * can have on a single file descriptor.  */
-#define MAXREDIRECT 16
+/* The number of fds space is allocated for  *
+ * each time a multio must increase in size. */
+#define MULTIOUNIT 8
 
 /* A multio is a list of fds associated with a certain fd.       *
  * Thus if you do "foo >bar >ble", the multio for fd 1 will have *
@@ -494,14 +499,9 @@ struct multio {
     int ct;			/* # of redirections on this fd                 */
     int rflag;			/* 0 if open for reading, 1 if open for writing */
     int pipe;			/* fd of pipe if ct > 1                         */
-    int fds[MAXREDIRECT];	/* list of src/dests redirected to/from this fd */
+    int fds[MULTIOUNIT];	/* list of src/dests redirected to/from this fd */
 };
 
-struct fdpair {
-    int fd1, fd2;
-};
-
-
 /* variable assignment tree element */
 
 struct varasg {
@@ -598,7 +598,6 @@ struct execstack {
     int list_pipe_child;
     int list_pipe_job;
     char list_pipe_text[JOBTEXTSIZE];
-    LinkList last_file_list;
     int lastval;
     int noeval;
     int badcshglob;
@@ -607,6 +606,7 @@ struct execstack {
     int trapreturn;
     int noerrs;
     int subsh_close;
+    char *underscore;
 };
 
 struct heredocs {
@@ -900,11 +900,6 @@ struct histent {
 #define STOPHIST (stophist += 4);
 #define ALLOWHIST (stophist -= 4);
 
-#define MAXAL 20		/* maximum number of aliases expanded at once */
-
-#define ALSTAT_MORE 1		/* last alias ended with ' '      */
-#define ALSTAT_JUNK 2		/* don't put word in history List */
-
 #define HISTFLAG_DONE   1
 #define HISTFLAG_NOEXEC 2
 #define HISTFLAG_RECALL 4
@@ -1246,8 +1241,6 @@ struct ttyinfo {
 #define txtchangeisset(X)	(txtchange & (X))
 #define txtchangeset(X, Y)	(txtchange |= (X), txtchange &= ~(Y))
 
-#define COLUMNS      (columns < 1 ? 80 : columns)
-
 /****************************************/
 /* Definitions for the %_ prompt escape */
 /****************************************/
